CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

 
  PinSD_CS        = 4             ' Pin for SD card
  PinSD_DI        = 5            ' Pin for SD card
  PinSD_CLK       = 6            ' Pin for SD card
  PinSD_DO        = 7            ' Pin for SD card
  PinSD_33        = 1
  PinSD_VSS       = 0
  PinSerGPS       = 9             ' Pin for GPS data (NB, this is not explicitly referred to in this code)
  PinSerFF        = 10
  PinLedGPS    = 11
  PinLedFF     = 12
  PinLedWrite  = 13
  PinFlo       = 14

  comma           = 0
  CR              = 1
OBJ
   sd            :               "fsrw"
   eeprom        :               "Propeller Eeprom"
   serGPS        :               "Simple_Serial"
   serFF         :               "Simple_Serial"
   num           :               "Simple_Numbers"

   debug         :               "SimpleDebug"
VAR 
   long FloCount
   long filename
   long stackGPS[80]
   long stackFF[80]
   long stackFlow[80]
   long stackMain[80]
   byte Dataout[64]
   byte calib
   byte flagGPS
   byte flagFF
   byte dataGPS[40]
   byte dataFF[40]
   long cntAsteriskGPS
   long cntAsteriskFF
   
pub main |r , conter, ctq, q, pwr, ftr, ct, tr
' r     - is used to store values returned from other methods.
' ctq   - used in the calculation of the calibration constant.
' q     - used in the calculation of the calibration constant.
' pwr   - used in the calculation of the calibration constant.
' ftr   - used in the calculation of the calibration constant.

waitcnt(cnt+160_000_000)
debug.start(9600)
debug.dec(1)  

                                     ' Warning LED
dira[PinLedFF]~~



dira[1] := 1
outa[1] := 0

dira[0] := 0
ina[0] := 1 


'============================================
debug.str(string("Opened. Set-up pins."))
debug.putc(13)
'============================================

r:= sd.mount_explicit(PinSD_DO, PinSD_CLK, PinSD_DI, PinSD_CS)
'============================================
debug.str(string("Mounted"))
debug.putc(13)
'===========================================

startWrite 


'cognew(ReadFF,    @stackFF)
'cognew(Flow,      @stackFlow)
r := cognew(MainLoop, @StackMain)                                                     ' Start the cog dedicated to the monitoring of clicks from the fuel
                                                                                 ' flow sensor.                        

'============================================
'debug.str(string("Cogs started..."))
debug.putc(13)
debug.putc(13) 
debug.dec(r)
debug.putc(13)
debug.putc(13) 
'============================================

'
r:=cognew(ReadGPS,   @stackGPS)

'============================================
debug.putc(13)
debug.putc(13) 
debug.dec(r)
debug.putc(13)
debug.putc(13)
'============================================
  
PUB MainLoop
dira[PinLedWrite]~~
repeat                                                                          ' This is the main loops, which is repeated ad infinitum
  
 {{ if flagGPS := 1
      callWriteByte(dataGPS, cntAsteriskGPS, comma)
  else
      callWriteByte(emptyGPS, 8, comma)  
   }}
  !outa[pinLedWrite] 
  
    
   



  waitcnt(cnt + 10_000_000)
  'flagGPS := 0 
  'tr := cnt
  'repeat until  (cnt - tr >= 80_000_000)

PUB ReadGPS | byteCnt, delayCnt, r, t
dira[PinLedGPS]~~  


repeat 5

  
  !outa[pinLedGPS]
  waitcnt(cnt + 40_000_000)

 
                       

PUB ReadFF
'serFF.start(PinSerFF, 16, 3, 4800)                                               ' Start the [two-way] RS232 communications with ELM327
'serFF.rxflush

PUB writeData                                                                 

  
PUB Flow
  ' This method runs in its own cog and is responsible for tracking the number of pulses from the flow
  '     sensor. The flow sensor is common-collector and requires a resistor.

waitpne(|< PinFlo, |< PinFlo, 0)                                              ' Initial setup.

 repeat                                                                         ' Repeat ad infintum...
    waitpeq(|<PinFlo, |< PinFlo, 0)                                           ' Check the pin to which the flow sensor is connected. Wait until it is HIGH
    FloCount := FloCount + calib                                                ' Update the fuel used variable 'FloCount' with the calibration constant.
    waitpne(|< PinFlo, |< PinFlo, 0)                                          ' Wait until the pin has changed state again.

                                                                                ' Rinse and repeat...                                                                                
    

PUB callWriteByte(v, ln, a)                                                                             ' Move data to the SD card by using the dataOut buffer.
  bytemove(@dataOut, v, ln)                                                
  sd.pwrite(@dataOut,ln)
  if a == comma
    writeComma
  else
    sd.pputc(13)
       
PUB startWrite | Fi, L
  fileName++                                                                                            ' Increment the filename number
  eeprom.VarBackup(@fileName, @fileName)                                                            ' Update the filename in the EEPROM so that it is non volatile

  Fi := num.dec(fileName)                                                                               ' Convert this to a decimal string
  L  := StrSize(Fi)                                                                                     ' Get the size of this string.
   
  byte[Fi+L]   := "."                                                                                   ' Add ".csv" to the file name. This will mean that the
  byte[Fi+L+1] := "C"                                                                                   '      filenames will be 1.csv, 2.csv, etc.
  byte[Fi+L+2] := "S"
  byte[Fi+L+3] := "V"
  byte[Fi+L+4] := 0                                                                                     ' Zero required to terminate the string. 
  sd.popen(Fi, "a")
  
  
  
  waitcnt(80_000_000 + cnt)

PUB writeComma
' When called, this method writes a comma (ASCII 44) to the SD card. Named to make it
  '    easy to remember when.
  sd.pputc(44)
DAT
emptyGPS byte ",,,,,,", 0
emptyFF  byte ",",0